@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        main.lua
@@ -30,7 +30,7 @@ import("core.cache.localcache")
 import("scangen")
 import("menuconf", {alias = "menuconf_show"})
 import("configfiles", {alias = "generate_configfiles"})
-import("private.action.require.check", {alias = "check_packages"})
+import("private.action.require.register", {alias = "register_packages"})
 import("private.action.require.install", {alias = "install_packages"})
 import("private.service.remote_build.action", {alias = "remote_build_action"})
 
@@ -51,6 +51,7 @@ function _option_filter(name)
     ,   export      = true
     ,   import      = true
     ,   check       = true
+    ,   menu        = true
     }
     return not options[name]
 end
@@ -94,26 +95,30 @@ function _need_check(changed)
     if not changed then
         if os.mtime(path.join(os.programdir(), "core", "main.lua")) > os.mtime(config.filepath()) then
             changed = true
+            os.touch(config.filepath(), {mtime = os.time()})
         end
     end
     return changed
 end
 
 -- check target
-function _check_target(target)
-    for _, depname in ipairs(target:get("deps")) do
-        assert(depname ~= target:name(), "the target(%s) cannot depend self!", depname)
-        local deptarget = project.target(depname)
-        assert(deptarget, "unknown target(%s) for %s.deps!", depname, target:name())
-        _check_target(deptarget)
+function _check_target(target, checked_targets)
+    if not checked_targets[target:fullname()] then
+        checked_targets[target:fullname()] = target
+        for _, depname in ipairs(target:get("deps")) do
+            local deptarget = project.target(depname, {namespace = target:namespace()})
+            assert(deptarget, "unknown target(%s) for %s.deps!", depname, target:fullname())
+            _check_target(deptarget, checked_targets)
+        end
     end
 end
 
 -- check targets
 function _check_targets()
     assert(not project.is_loaded(), "project and targets may have been loaded early!")
+    local checked_targets = {}
     for _, target in pairs(project.targets()) do
-        _check_target(target)
+        _check_target(target, checked_targets)
     end
 end
 
@@ -354,7 +359,6 @@ force to build in current directory via run `xmake -P .`]], os.projectdir())
         localcache.clear("option")
         localcache.clear("package")
         localcache.clear("toolchain")
-        localcache.clear("cxxmodules")
         localcache.set("config", "recheck", true)
         localcache.save()
 
@@ -368,9 +372,14 @@ force to build in current directory via run `xmake -P .`]], os.projectdir())
     end
 
     -- translate the build directory
-    local buildir = config.get("buildir")
-    if buildir and path.is_absolute(buildir) then
-        config.set("buildir", path.relative(buildir, project.directory()), {readonly = true, force = true})
+    local builddir = config.get("builddir")
+    if config.get("buildir") then
+        wprint("`xmake f --buildir=` has been deprecated, please use `xmake f -o/--builddir=`")
+        builddir = config.get("buildir")
+        config.set("builddir", builddir, {readonly = true, force = true})
+    end
+    if builddir and path.is_absolute(builddir) then
+        config.set("builddir", path.relative(builddir, project.directory()), {readonly = true, force = true})
     end
 
     -- only config for building project using third-party buildsystem
@@ -379,13 +388,13 @@ force to build in current directory via run `xmake -P .`]], os.projectdir())
         -- check configs
         _check_configs()
 
-        -- install and update packages
+        -- install and register packages
         local require_enable = option.boolean(option.get("require"))
         if (recheck or require_enable) then
             if require_enable ~= false then
                 install_packages()
             else
-                check_packages()
+                register_packages()
             end
         end
 
