@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        scheduler.lua
@@ -179,8 +179,10 @@ function scheduler:_poller_resume_co(co, events)
         events = poller.EV_POLLER_ERROR
     end
 
-    -- this coroutine must be suspended
-    assert(co:is_suspended())
+    -- this coroutine must be suspended, (maybe also dead)
+    if not co:is_suspended() then
+        return false, string.format("%s cannot be resumed, status: %s", co, co:status())
+    end
 
     -- resume this coroutine task
     co:waitobj_set(nil)
@@ -434,7 +436,29 @@ end
 
 -- resume the given coroutine
 function scheduler:co_resume(co, ...)
-    return coroutine.resume(co:thread(), ...)
+
+    -- do resume
+    local ok, errors = coroutine.resume(co:thread(), ...)
+
+    local running = self:co_running()
+    if running then
+
+        -- has the current directory been changed? restore it
+        local curdir = self._CO_CURDIR_HASH
+        local olddir = self._CO_CURDIRS and self._CO_CURDIRS[running] or nil
+        if olddir and curdir ~= olddir[1] then -- hash changed?
+            os.cd(olddir[2])
+        end
+
+        -- has the current environments been changed? restore it
+        local curenvs = self._CO_CURENVS_HASH
+        local oldenvs = self._CO_CURENVS and self._CO_CURENVS[running] or nil
+        if oldenvs and curenvs ~= oldenvs[1] and running:is_isolated() then -- hash changed?
+            os.setenvs(oldenvs[2])
+        end
+    end
+
+    return ok, errors
 end
 
 -- suspend the current coroutine
@@ -443,7 +467,7 @@ function scheduler:co_suspend(...)
     -- suspend it
     local results = table.pack(coroutine.yield(...))
 
-    -- if the current directory has been changed? restore it
+    -- has the current directory been changed? restore it
     local running = assert(self:co_running())
     local curdir = self._CO_CURDIR_HASH
     local olddir = self._CO_CURDIRS and self._CO_CURDIRS[running] or nil
@@ -451,7 +475,7 @@ function scheduler:co_suspend(...)
         os.cd(olddir[2])
     end
 
-    -- if the current environments has been changed? restore it
+    -- has the current environments been changed? restore it
     local curenvs = self._CO_CURENVS_HASH
     local oldenvs = self._CO_CURENVS and self._CO_CURENVS[running] or nil
     if oldenvs and curenvs ~= oldenvs[1] and running:is_isolated() then -- hash changed?
@@ -470,7 +494,7 @@ end
 -- sleep some times (ms)
 function scheduler:co_sleep(ms)
 
-    -- we need not do sleep
+    -- we don't need to sleep
     if ms == 0 then
         return true
     end
@@ -499,6 +523,87 @@ function scheduler:co_sleep(ms)
     return true
 end
 
+-- lock the current coroutine
+function scheduler:co_lock(lockname)
+
+    -- get the running coroutine
+    local running = self:co_running()
+    if not running then
+        return false, "we must call co_lock() in coroutine with scheduler!"
+    end
+
+    -- is stopped?
+    if not self._STARTED then
+        return false, "the scheduler is stopped!"
+    end
+
+    -- do lock
+    local co_locked_tasks = self._CO_LOCKED_TASKS
+    if co_locked_tasks == nil then
+        co_locked_tasks = {}
+        self._CO_LOCKED_TASKS = co_locked_tasks
+    end
+    while true do
+
+        -- try to lock it
+        if co_locked_tasks[lockname] == nil then
+            co_locked_tasks[lockname] = running
+            return true
+        -- has been locked by the current coroutine
+        elseif co_locked_tasks[lockname] == running then
+            return true
+        end
+
+        -- register timeout task to timer
+        local function timer_callback (cancel)
+            if co_locked_tasks[lockname] == nil then
+                if running:is_suspended() then
+                    return self:co_resume(running)
+                end
+            else
+                self:_timer():post(timer_callback, 500)
+            end
+            return true
+        end
+        self:_timer():post(timer_callback, 500)
+
+        -- wait
+        self:co_suspend()
+    end
+    return true
+end
+
+-- unlock the current coroutine
+function scheduler:co_unlock(lockname)
+
+    -- get the running coroutine
+    local running = self:co_running()
+    if not running then
+        return false, "we must call co_unlock() in coroutine with scheduler!"
+    end
+
+    -- is stopped?
+    if not self._STARTED then
+        return false, "the scheduler is stopped!"
+    end
+
+    -- do unlock
+    local co_locked_tasks = self._CO_LOCKED_TASKS
+    if co_locked_tasks == nil then
+        co_locked_tasks = {}
+        self._CO_LOCKED_TASKS = co_locked_tasks
+    end
+    if co_locked_tasks[lockname] == nil then
+        return false, string.format("we need to call lock(%s) first before calling unlock(%s)", lockname, lockname)
+    end
+    if co_locked_tasks[lockname] == running then
+        co_locked_tasks[lockname] = nil
+    else
+        return false, string.format("unlock(%s) is called in other %s", lockname, running)
+    end
+    return true
+end
+
 -- get the given coroutine group
 function scheduler:co_group(name)
     return self._CO_GROUPS and self._CO_GROUPS[name]
@@ -970,20 +1075,16 @@ function scheduler:runloop()
                 if eventfunc then
                     ok, errors = eventfunc(self, obj, objevents)
                     if not ok then
-                        -- TODO
-                        --
                         -- This causes a direct exit from the entire runloop and
                         -- a quick escape from nested try-catch blocks and coroutines groups.
                         --
                         -- So some try-catch cannot catch these errors, such as when a build fails (in build group).
                         -- @see https://github.com/xmake-io/xmake/issues/3401
                         --
-                        -- In theory, we should handle it better. For example, if there is a group that is waiting,
-                        -- we should notify the other concurrent threads to exit quickly and then let the group concurrent threads to throw the error.
-                        -- That way the outside try-catch can continue to catch it.
+                        -- We should catch it in coroutines and re-throw it outside scheduler,
+                        -- it will avoid co_resume to get and return this error.
                         --
-                        -- But implementing it is more complicated and I haven't come up with a solution to let other concurrent processes exit quickly,
-                        -- especially if the child process is waiting and we need to notify it of the end quickly as well.
+                        -- e.g. we can see runjobs.lua implementation
                         break
                     end
                 end
