@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        builder.lua
@@ -145,15 +145,12 @@ end
 -- add_cflags("", {interface = true})
 --
 function builder:_inherit_flags_from_targetdeps(flags, target)
-    local orderdeps = target:orderdeps()
+    local orderdeps = target:orderdeps({inherit = true})
     local total = #orderdeps
     for idx, _ in ipairs(orderdeps) do
         local dep = orderdeps[total + 1 - idx]
-        local depinherit = target:extraconf("deps", dep:name(), "inherit")
-        if depinherit == nil or depinherit then
-            for _, flagkind in ipairs(self:_flagkinds()) do
-                self:_add_flags_from_flagkind(flags, dep, flagkind, {interface = true})
-            end
+        for _, flagkind in ipairs(self:_flagkinds()) do
+            self:_add_flags_from_flagkind(flags, dep, flagkind, {interface = true})
         end
     end
 end
@@ -213,18 +210,33 @@ end
 
 -- add flags from the target options
 function builder:_add_flags_from_targetopts(flags, target)
-    for _, opt in ipairs(target:orderopts()) do
-        for _, flagkind in ipairs(self:_flagkinds()) do
-            self:_add_flags_from_flagkind(flags, opt, flagkind)
+    for _, flagkind in ipairs(self:_flagkinds()) do
+        local result = target:get_from(flagkind, "option::*")
+        if result then
+            for _, values in ipairs(table.wrap(result)) do
+                table.join2(flags, self:_mapflags(values, flagkind, target))
+            end
         end
     end
 end
 
 -- add flags from the target packages
 function builder:_add_flags_from_targetpkgs(flags, target)
-    for _, pkg in ipairs(target:orderpkgs()) do
-        for _, flagkind in ipairs(self:_flagkinds()) do
-            table.join2(flags, self:_mapflags(pkg:get(flagkind), flagkind, target))
+    local kind = self:kind()
+    for _, flagkind in ipairs(self:_flagkinds()) do
+        -- attempt to add special lanugage flags from package first, e.g. gcldflags, dcarflags
+        -- @see https://github.com/xmake-io/xmake-repo/issues/5255
+        local result
+        if kind:endswith("ld") or kind:endswith("sh") then
+            result = target:get_from(kind .. "flags", "package::*")
+        end
+        if not result then
+            result = target:get_from(flagkind, "package::*")
+        end
+        if result then
+            for _, values in ipairs(table.wrap(result)) do
+                table.join2(flags, self:_mapflags(values, flagkind, target))
+            end
         end
     end
 end
@@ -306,6 +318,9 @@ function builder:_add_flags_from_argument(flags, target, args)
             return values, extras
         end,
         toolchain = function (name)
+            if target and target.toolconfig then
+                return target:toolconfig(name)
+            end
             local plat, arch
             if target and target.plat then
                 plat = target:plat()
@@ -391,7 +406,7 @@ function builder:_add_items_from_target(items, name, opt)
         if result then
             for idx, values in ipairs(result) do
                 local source = sources[idx]
-                local extras = target:extraconf_from(source, name)
+                local extras = target:extraconf_from(name, source)
                 values = table.wrap(values)
                 if values and #values > 0 then
                     table.insert(items, {
@@ -653,11 +668,14 @@ function builder:_sort_links_of_items(items, opt)
             gh:add_edge(k, v)
         end
         if not gh:empty() then
-            local cycle = gh:find_cycle()
-            if cycle then
-                utils.warning("cycle links found in add_linkorders(): %s", table.concat(cycle, " -> "))
+            local has_cycle
+            links, has_cycle = gh:topo_sort()
+            if has_cycle then
+                local cycle = gh:find_cycle()
+                if cycle then
+                    utils.warning("cycle links found in add_linkorders(): %s", table.concat(cycle, " -> "))
+                end
             end
-            links = gh:topological_sort()
         end
     end
 
@@ -770,12 +788,32 @@ end
 function builder:map_flags(name, values, opt)
     local flags  = {}
     local mapper = self:_tool()["nf_" .. name]
+    local multival = false
+    if name:endswith("s") then
+        multival = true
+    elseif not mapper then
+        mapper = self:_tool()["nf_" .. name .. "s"]
+        if mapper then
+            multival = true
+        end
+    end
     if mapper then
         opt = opt or {}
-        for _, value in ipairs(table.wrap(values)) do
-            local flag = mapper(self:_tool(), value, opt.target, opt.targetkind)
-            if flag and flag ~= "" and (not opt.check or self:has_flags(flag)) then
-                table.join2(flags, flag)
+        if multival then
+            local extra = self:_extraconf(opt.extras, values)
+            local results = mapper(self:_tool(), values, {target = opt.target, targetkind = opt.targetkind, extra = extra})
+            for _, flag in ipairs(table.wrap(results)) do
+                if flag and flag ~= "" and (not opt.check or self:has_flags(flag)) then
+                    table.insert(flags, flag)
+                end
+            end
+        else
+            for _, value in ipairs(table.wrap(values)) do
+                local extra = self:_extraconf(opt.extras, value)
+                local flag = mapper(self:_tool(), value, {target = opt.target, targetkind = opt.targetkind, extra = extra})
+                if flag and flag ~= "" and (not opt.check or self:has_flags(flag)) then
+                    table.join2(flags, flag)
+                end
             end
         end
     end
