@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        rc.lua
@@ -79,22 +79,20 @@ function nf_sysincludedir(self, dir)
 end
 
 -- make the compile arguments list
-function compargv(self, sourcefile, objectfile, flags)
+function compargv(self, sourcefile, objectfile, flags, opt)
     return self:program(), table.join(flags, "-Fo" .. objectfile, sourcefile)
 end
 
 -- compile the source file
-function compile(self, sourcefile, objectfile, dependinfo, flags)
-
-    -- ensure the object directory
+function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
+    opt = opt or {}
     os.mkdir(path.directory(objectfile))
 
-    -- compile it
     try
     {
         function ()
             -- @note we don't need to use vstool.iorunv to enable unicode output for rc.exe
-            local program, argv = compargv(self, sourcefile, objectfile, flags)
+            local program, argv = compargv(self, sourcefile, objectfile, flags, opt)
             local outdata, errdata = os.iorunv(program, argv, {envs = self:runenvs()})
             return (outdata or "") .. (errdata or "")
         end,
@@ -124,7 +122,12 @@ function compile(self, sourcefile, objectfile, dependinfo, flags)
 
     -- try to use cl.exe to parse includes, but cl.exe maybe not exists in masm32 sdk
     -- @see https://github.com/xmake-io/xmake/issues/2562
-    local cl = self:toolchain():tool("cxx")
+    local cl
+    if opt.target then
+        cl = opt.target:tool("cxx")
+    elseif self:toolchain() then
+        cl = self:toolchain():tool("cxx")
+    end
     if cl then
         local outfile = os.tmpfile() .. ".rc.out"
         local errfile = os.tmpfile() .. ".rc.err"
@@ -154,7 +157,8 @@ function compile(self, sourcefile, objectfile, dependinfo, flags)
             end
             file:close()
             if dependinfo then
-                dependinfo.depfiles_rc = depfiles_rc
+                dependinfo.depfiles_format = "rc"
+                dependinfo.depfiles = depfiles_rc
             end
         end
         os.tryrm(outfile)
