@@ -12,16 +12,19 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        find_vstudio.lua
 --
 
 -- imports
+import("core.base.option")
+import("core.base.semver")
 import("core.project.config")
 import("lib.detect.find_file")
 import("lib.detect.find_tool")
+import("lib.detect.find_directory")
 import("core.cache.global_detectcache")
 
 -- init vc variables
@@ -97,8 +100,142 @@ function get_vcvars()
     return realvcvars
 end
 
+function find_build_tools(opt)
+    opt = opt or {}
+
+    local sdkdir = opt.sdkdir
+    if not sdkdir or not os.isdir(sdkdir) then
+        return
+    end
+
+    local variables = {}
+    local VCInstallDir = path.join(sdkdir, "VC")
+    local VCToolsVersion = opt.vs_toolset
+    if not VCToolsVersion or not os.isdir(path.join(VCInstallDir, "Tools/MSVC", VCToolsVersion)) then
+        -- https://github.com/xmake-io/xmake/issues/6159
+        local latest_toolset
+        for _, dir in ipairs(os.dirs(path.join(sdkdir, "VC/Tools/MSVC/*"))) do
+            local toolset = path.filename(dir)
+            if not latest_toolset or semver.compare(toolset, latest_toolset) > 0 then
+                latest_toolset = toolset
+            end
+        end
+        if latest_toolset then
+            VCToolsVersion = latest_toolset
+        else
+            return
+        end
+    end
+    variables.VCInstallDir = VCInstallDir
+    variables.VCToolsVersion = VCToolsVersion
+    variables.VCToolsInstallDir = path.join(VCInstallDir, "Tools/MSVC", VCToolsVersion)
+
+    local WindowsSDKVersion
+    local vs_sdkver = opt.vs_sdkver
+    if vs_sdkver and os.isdir(path.join(sdkdir, "Windows Kits/10/Lib", vs_sdkver)) then
+        WindowsSDKVersion = vs_sdkver
+    else
+        local dir = find_directory("10*", path.join(sdkdir, "Windows Kits/10/Lib"))
+        if dir then
+            WindowsSDKVersion = path.filename(dir)
+        else
+            return
+        end
+    end
+    variables.WindowsSDKVersion = WindowsSDKVersion
+    variables.WindowsSdkDir = path.join(sdkdir, "Windows Kits/10")
+    variables.WindowsSdkBinPath = path.join(variables.WindowsSdkDir, "bin")
+    variables.WindowsSdkVerBinPath = path.join(variables.WindowsSdkBinPath, WindowsSDKVersion)
+    variables.ExtensionSdkDir = path.join(variables.WindowsSdkDir, "ExtensionSdkDir")
+    variables.UCRTVersion = WindowsSDKVersion
+    variables.UniversalCRTSdkDir = variables.WindowsSdkDir
+
+    local includedirs = {
+        path.join(variables.VCToolsInstallDir, "include"),
+        path.join(variables.VCToolsInstallDir, "atlmfc", "include"),
+        path.join(variables.WindowsSdkDir, "Include", WindowsSDKVersion, "ucrt"),
+        path.join(variables.WindowsSdkDir, "Include", WindowsSDKVersion, "shared"),
+        path.join(variables.WindowsSdkDir, "Include", WindowsSDKVersion, "um"),
+        path.join(variables.WindowsSdkDir, "Include", WindowsSDKVersion, "winrt"),
+        path.join(variables.WindowsSdkDir, "Include", WindowsSDKVersion, "cppwinrt"),
+    }
+
+    local linkdirs = {
+        path.join(variables.VCToolsInstallDir, "lib"),
+        path.join(variables.VCToolsInstallDir, "atlmfc", "lib"),
+        path.join(variables.WindowsSdkDir, "Lib", WindowsSDKVersion, "ucrt"),
+        path.join(variables.WindowsSdkDir, "Lib", WindowsSDKVersion, "um"),
+        path.join(variables.WindowsSdkDir, "Lib", WindowsSDKVersion, "km"),
+    }
+
+    local archs = {
+        "x86",
+        "x64",
+        "arm",
+        "arm64",
+    }
+
+    local vcvarsall = {}
+    for _, target_arch in ipairs(archs) do
+        local lib = {}
+        for _, lib_dir in ipairs(linkdirs) do
+            local dir = path.join(lib_dir, target_arch)
+            if os.isdir(dir) then
+                table.insert(lib, dir)
+            end
+        end
+
+        if #lib ~= 0 then
+            local vcvars = {
+                BUILD_TOOLS_ROOT = sdkdir,
+                VSInstallDir = sdkdir,
+
+                -- vs runs in a windows ctx, so the envsep is always ";"
+                INCLUDE = path.joinenv(includedirs, ';'),
+                LIB = path.joinenv(lib, ';'),
+
+                VSCMD_ARG_HOST_ARCH = "x64",
+
+                VCInstallDir = variables.VCInstallDir,
+                VCToolsVersion = variables.VCToolsVersion,
+                VCToolsInstallDir = variables.VCToolsInstallDir,
+
+                WindowsSDKVersion = variables.WindowsSDKVersion,
+                WindowsSdkDir = variables.WindowsSdkDir,
+                WindowsSdkBinPath = variables.WindowsSdkBinPath,
+                WindowsSdkVerBinPath = variables.WindowsSdkVerBinPath,
+                ExtensionSdkDir = variables.ExtensionSdkDir,
+                UCRTVersion = variables.UCRTVersion,
+                UniversalCRTSdkDir = variables.UniversalCRTSdkDir,
+            }
+
+            local build_tools_bin = {}
+            local host_dir = "Host" .. vcvars.VSCMD_ARG_HOST_ARCH
+            if is_host("windows") then
+                table.insert(build_tools_bin, path.join(vcvars.VCToolsInstallDir, "bin", host_dir, target_arch))
+                table.insert(build_tools_bin, path.join(vcvars.WindowsSdkDir, "bin", WindowsSDKVersion))
+                table.insert(build_tools_bin, path.join(vcvars.WindowsSdkDir, "bin", WindowsSDKVersion, "ucrt"))
+            elseif is_host("linux") then
+                -- for msvc-wine
+                table.insert(build_tools_bin, path.join(sdkdir, "bin", target_arch))
+            end
+
+            vcvars.VSCMD_ARG_TGT_ARCH = target_arch
+            vcvars.BUILD_TOOLS_BIN = path.joinenv(build_tools_bin)
+
+            local PATH = build_tools_bin
+            table.join2(PATH, path.splitenv(os.getenv("PATH")))
+            vcvars.PATH = path.joinenv(PATH)
+
+            vcvarsall[target_arch] = vcvars
+        end
+    end
+
+    return vcvarsall
+end
+
 -- load vcvarsall environment variables
-function _load_vcvarsall(vcvarsall, vsver, arch, opt)
+function _load_vcvarsall_impl(vcvarsall, vsver, arch, opt)
     opt = opt or {}
 
     -- is VsDevCmd.bat?
@@ -118,17 +255,33 @@ function _load_vcvarsall(vcvarsall, vsver, arch, opt)
     if vsver and tonumber(vsver) >= 16 then
         file:print("set VSCMD_SKIP_SENDTELEMETRY=yes")
     end
+    local host_arch = os.arch()
     if is_vsdevcmd then
-        if opt.vcvars_ver then
-            file:print("call \"%s\" -arch=%s -winsdk=%s -vcvars_ver=%s > nul", vcvarsall, arch, opt.sdkver and opt.sdkver or "", opt.vcvars_ver)
+        if vsver and tonumber(vsver) >= 16 then
+            if opt.toolset then
+                file:print("call \"%s\" -host_arch=%s -arch=%s -winsdk=%s -vcvars_ver=%s > nul", vcvarsall, host_arch, arch, opt.sdkver or "", opt.toolset or "")
+            else
+                file:print("call \"%s\" -host_arch=%s -arch=%s -winsdk=%s > nul", vcvarsall, host_arch, arch, opt.sdkver or "")
+            end
         else
-            file:print("call \"%s\" -arch=%s -winsdk=%s > nul", vcvarsall, arch, opt.sdkver and opt.sdkver or "")
+            if opt.toolset then
+                file:print("call \"%s\" -arch=%s -winsdk=%s -vcvars_ver=%s > nul", vcvarsall, arch, opt.sdkver or "", opt.toolset or "")
+            else
+                file:print("call \"%s\" -arch=%s -winsdk=%s > nul", vcvarsall, arch, opt.sdkver or "")
+            end
         end
     else
-        if opt.vcvars_ver then
-            file:print("call \"%s\" %s %s -vcvars_ver=%s > nul", vcvarsall, arch, opt.sdkver and opt.sdkver or "", opt.vcvars_ver)
+        -- @see https://github.com/xmake-io/xmake/issues/5077
+        if vsver and tonumber(vsver) >= 16 and host_arch ~= arch then
+            if host_arch == "x64" then
+                host_arch = "amd64"
+            end
+            arch = host_arch .. "_" .. arch
+        end
+        if opt.toolset then
+            file:print("call \"%s\" %s %s -vcvars_ver=%s > nul", vcvarsall, arch, opt.sdkver or "", opt.toolset or "")
         else
-            file:print("call \"%s\" %s %s > nul", vcvarsall, arch, opt.sdkver and opt.sdkver or "")
+            file:print("call \"%s\" %s %s > nul", vcvarsall, arch, opt.sdkver or "")
         end
     end
     for idx, var in ipairs(get_vcvars()) do
@@ -137,7 +290,10 @@ function _load_vcvarsall(vcvarsall, vsver, arch, opt)
     file:close()
 
     -- run genvcvars.bat
-    local outdata = try {function () return os.iorun(genvcvars_bat) end}
+    local outdata, errdata = try {function () return os.iorun(genvcvars_bat) end}
+    if errdata and #errdata > 0 and option.get("verbose") and option.get("diagnosis") then
+        cprint("${color.warning}checkinfo: ${clear dim}get vcvars error: %s", errdata)
+    end
     if not outdata then
         return
     end
@@ -170,6 +326,16 @@ function _load_vcvarsall(vcvarsall, vsver, arch, opt)
         if WindowsSDKVersion ~= "" then
             variables["WindowsSDKVersion"] = WindowsSDKVersion
         end
+    else
+        -- sometimes the variable `WindowsSDKVersion` is not available
+        -- then parse it from `WindowsSdkBinPath`, such as: `C:\\Program Files (x86)\\Windows Kits\\8.1\\bin`
+        local WindowsSdkBinPath = variables["WindowsSdkBinPath"]
+        if WindowsSdkBinPath then
+            WindowsSDKVersion = string.match(WindowsSdkBinPath, "\\(%d+%.%d+)\\bin$")
+            if WindowsSDKVersion then
+                variables["WindowsSDKVersion"] = WindowsSDKVersion
+            end
+        end
     end
 
     -- fix UCRTVersion
@@ -204,6 +370,44 @@ function _load_vcvarsall(vcvarsall, vsver, arch, opt)
     return variables
 end
 
+-- strip toolset version, e.g. 14.16.27023 -> 14.16
+function _strip_toolset_ver(vs_toolset)
+    local version = semver.new(vs_toolset)
+    if version then
+        return version:major() .. "." .. version:minor()
+    end
+    return vs_toolset
+end
+
+function _load_vcvarsall(vcvarsall, vsver, arch, opt)
+    opt = opt or {}
+    local vs_toolset = opt.toolset or opt.vcvars_ver
+    if vs_toolset then
+        opt.toolset = _strip_toolset_ver(vs_toolset)
+    end
+    local result = _load_vcvarsall_impl(vcvarsall, vsver, arch, opt)
+    if result and not vs_toolset then
+        -- if no vs toolset version is specified, we default to the latest version.
+        -- https://github.com/xmake-io/xmake/issues/6159
+        local latest_toolset
+        local VCToolsVersion = result.VCToolsVersion
+        local VCInstallDir = result.VCInstallDir
+        if VCToolsVersion and VCInstallDir then
+            for _, dir in ipairs(os.dirs(path.join(VCInstallDir, "Tools/MSVC/*"))) do
+                local toolset = path.filename(dir)
+                if not latest_toolset or semver.compare(toolset, latest_toolset) > 0 then
+                    latest_toolset = toolset
+                end
+            end
+        end
+        if latest_toolset and VCToolsVersion and semver.compare(latest_toolset, VCToolsVersion) > 0 then
+            opt.toolset = _strip_toolset_ver(latest_toolset)
+            result = _load_vcvarsall_impl(vcvarsall, vsver, arch, opt)
+        end
+    end
+    return result
+end
+
 -- find vstudio for msvc
 function _find_vstudio(opt)
     opt = opt or {}
@@ -250,24 +454,25 @@ function _find_vstudio(opt)
         if vcvarsall and os.isfile(vcvarsall) and vsvers[VisualStudioVersion] then
 
             -- load vcvarsall
-            local vcvarsall_x86 = _load_vcvarsall(vcvarsall, VisualStudioVersion, "x86", opt)
-            local vcvarsall_x64 = _load_vcvarsall(vcvarsall, VisualStudioVersion, "x64", opt)
-
-            -- load vcvarsall for arm64
-            local arch
-            local arch_os = os.arch()
-            if arch_os == "x64" then
-                arch = "x64_arm64"
-            elseif arch_os == "x86" then
-                arch = "x86_arm64"
-            elseif arch_os == "arm64" then
-                arch = "arm64"
-            end
-            local vcvarsall_arm64 = arch and _load_vcvarsall(vcvarsall, version, arch, opt) or nil
+            local vcvarsall_x86     = _load_vcvarsall(vcvarsall, VisualStudioVersion, "x86", opt)
+            local vcvarsall_x64     = _load_vcvarsall(vcvarsall, VisualStudioVersion, "x64", opt)
+            local vcvarsall_arm     = _load_vcvarsall(vcvarsall, VisualStudioVersion, "arm", opt)
+            local vcvarsall_arm64   = _load_vcvarsall(vcvarsall, VisualStudioVersion, "arm64", opt)
+            local vcvarsall_arm64ec = vcvarsall_arm64
 
             -- save results
             local results = {}
-            results[vsvers[VisualStudioVersion]] = {version = VisualStudioVersion, vcvarsall_bat = vcvarsall, vcvarsall = {x86 = vcvarsall_x86, x64 = vcvarsall_x64, arm64 = vcvarsall_arm64}}
+            results[vsvers[VisualStudioVersion]] = {
+                version = VisualStudioVersion,
+                vcvarsall_bat = vcvarsall,
+                vcvarsall = {
+                    x86 = vcvarsall_x86,
+                    x64 = vcvarsall_x64,
+                    arm = vcvarsall_arm,
+                    arm64 = vcvarsall_arm64,
+                    arm64ec = vcvarsall_arm64ec
+                }
+            }
             return results
         end
     end
@@ -286,15 +491,17 @@ function _find_vstudio(opt)
         -- find VC install path (and aux build path) using `vswhere` (for version >= 15.0)
         -- * version > 15.0 eschews registry entries; but `vswhere` (included with version >= 15.2) can be used to find VC install path
         -- ref: https://github.com/Microsoft/vswhere/blob/master/README.md @@ https://archive.is/mEmdu
-        local vswhere_VCAuxiliaryBuildDir = nil
-        local vswhere_Common7ToolsDir = nil
+        local vswhere_VCAuxiliaryBuildDir = {}
+        local vswhere_Common7ToolsDir = {}
         if (tonumber(version) >= 15) and vswhere then
             local vswhere_vrange = format("%s,%s)", version, (version + 1))
             -- build tools: https://github.com/microsoft/vswhere/issues/22 @@ https://aka.ms/vs/workloads
             local result = os.iorunv(vswhere.program, {"-products", "*", "-prerelease", "-property", "installationpath", "-version", vswhere_vrange})
             if result then
-                vswhere_VCAuxiliaryBuildDir = path.join(result:trim(), "VC", "Auxiliary", "Build")
-                vswhere_Common7ToolsDir = path.join(result:trim(), "Common7", "Tools")
+                for _, vc_path in ipairs(result:split("\n")) do
+                    table.insert(vswhere_VCAuxiliaryBuildDir, path.join(vc_path:trim(), "VC", "Auxiliary", "Build"))
+                    table.insert(vswhere_Common7ToolsDir, path.join(vc_path:trim(), "Common7", "Tools"))
+                end
             end
         end
 
@@ -309,8 +516,17 @@ function _find_vstudio(opt)
         if vsenvs[version] then
             table.insert(paths, format("$(env %s)\\..\\..\\VC", vsenvs[version]))
         end
-        if vswhere_VCAuxiliaryBuildDir and os.isdir(vswhere_VCAuxiliaryBuildDir) then
-            table.insert(paths, 1, vswhere_VCAuxiliaryBuildDir)
+        
+        if vswhere_VCAuxiliaryBuildDir then
+            for _, vc_path in ipairs(vswhere_VCAuxiliaryBuildDir) do
+                if os.isdir(vc_path) then
+                    table.insert(paths, 1, vc_path)
+                end
+            end
+        end
+        if version == "6.0" and os.arch() == "x64" then
+            table.insert(paths, "$(reg HKEY_LOCAL_MACHINE\\SOFTWARE\\WOW6432Node\\Microsoft\\DevStudio\\6.0\\Products\\Microsoft Visual C++;ProductDir)\\Bin")
+            table.insert(paths, "$(reg HKEY_LOCAL_MACHINE\\SOFTWARE\\WOW6432Node\\Microsoft\\VisualStudio\\6.0\\Setup\\Microsoft Visual C++;ProductDir)\\Bin")
         end
 
         -- find vcvarsall.bat, vcvars32.bat for vs7.1
@@ -338,6 +554,7 @@ function _find_vstudio(opt)
                 table.insert(paths, path.join(logical_drive, "Program Files", "Microsoft Visual Studio " .. version, "VC"))
                 if version == "6.0" then
                     table.insert(paths, path.join(logical_drive, "Program Files", "Microsoft Visual Studio", "VC98", "Bin"))
+                    table.insert(paths, path.join(logical_drive, "Program Files (x86)", "Microsoft Visual Studio", "VC98", "Bin"))
                 end
             end
             vcvarsall = find_file("vcvarsall.bat", paths) or find_file("vcvars32.bat", paths)
@@ -347,31 +564,36 @@ function _find_vstudio(opt)
                 format("$(reg HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\VisualStudio\\SxS\\VS7;%s)\\Common7\\Tools", version),
                 format("$(reg HKEY_LOCAL_MACHINE\\SOFTWARE\\Wow6432Node\\Microsoft\\VisualStudio\\SxS\\VS7;%s)\\Common7\\Tools", version)
             }
-            if vswhere_Common7ToolsDir and os.isdir(vswhere_Common7ToolsDir) then
-                table.insert(paths, 1, vswhere_Common7ToolsDir)
+            if vswhere_Common7ToolsDir then
+                for _, vc_path in ipairs(vswhere_Common7ToolsDir) do
+                    if os.isdir(vc_path) then
+                        table.insert(paths, 1, vc_path)
+                    end
+                end
             end
             vcvarsall = find_file("VsDevCmd.bat", paths)
         end
         if vcvarsall then
 
             -- load vcvarsall
-            local vcvarsall_x86 = _load_vcvarsall(vcvarsall, version, "x86", opt)
-            local vcvarsall_x64 = _load_vcvarsall(vcvarsall, version, "x64", opt)
-
-            -- load vcvarsall for arm64
-            local arch
-            local arch_os = os.arch()
-            if arch_os == "x64" then
-                arch = "x64_arm64"
-            elseif arch_os == "x86" then
-                arch = "x86_arm64"
-            elseif arch_os == "arm64" then
-                arch = "arm64"
-            end
-            local vcvarsall_arm64 = arch and _load_vcvarsall(vcvarsall, version, arch, opt) or nil
+            local vcvarsall_x86     = _load_vcvarsall(vcvarsall, version, "x86", opt)
+            local vcvarsall_x64     = _load_vcvarsall(vcvarsall, version, "x64", opt)
+            local vcvarsall_arm     = _load_vcvarsall(vcvarsall, version, "arm", opt)
+            local vcvarsall_arm64   = _load_vcvarsall(vcvarsall, version, "arm64", opt)
+            local vcvarsall_arm64ec = vcvarsall_arm64
 
             -- save results
-            results[vsvers[version]] = {version = version, vcvarsall_bat = vcvarsall, vcvarsall = {x86 = vcvarsall_x86, x64 = vcvarsall_x64, arm64 = vcvarsall_arm64}}
+            results[vsvers[version]] = {
+                version = version,
+                vcvarsall_bat = vcvarsall,
+                vcvarsall = {
+                    x86 = vcvarsall_x86,
+                    x64 = vcvarsall_x64,
+                    arm = vcvarsall_arm,
+                    arm64 = vcvarsall_arm64,
+                    arm64ec = vcvarsall_arm64ec
+                }
+            }
         end
     end
     return results
@@ -429,7 +651,7 @@ end
 
 -- find vstudio environment
 --
--- @param opt   the options, e.g. {vcvars_ver = 14.0, sdkver = "10.0.15063.0"}
+-- @param opt   the options, e.g. {toolset = 14.0, sdkver = "10.0.15063.0"}
 --
 -- @return      { 2008 = {version = "9.0", vcvarsall = {x86 = {path = .., lib = .., include = ..}}}
 --              , 2017 = {version = "15.0", vcvarsall = {x64 = {path = .., lib = ..}}}}
@@ -443,8 +665,11 @@ function main(opt)
     end
 
     local key = "vstudio"
-    if opt.vcvars_ver then
-        key = key .. opt.vcvars_ver
+    if opt.toolset then
+        key = key .. opt.toolset
+    end
+    if opt.sdkver then
+        key = key .. opt.sdkver
     end
 
     -- attempt to get it from the global cache first
@@ -467,3 +692,4 @@ function main(opt)
     end
     return vstudio
 end
+
