@@ -12,7 +12,7 @@
 -- See the License for the specific language governing permissions and
 -- limitations under the License.
 --
--- Copyright (C) 2015-present, TBOOX Open Source Group.
+-- Copyright (C) 2015-present, Xmake Open Source Community.
 --
 -- @author      ruki
 -- @file        vs201x.lua
@@ -39,6 +39,8 @@ import("private.action.require.install", {alias = "install_requires"})
 import("private.action.run.runenvs")
 import("actions.config.configfiles", {alias = "generate_configfiles", rootdir = os.programdir()})
 import("private.utils.batchcmds")
+import("plugins.project.utils.target_cmds", {rootdir = os.programdir()})
+import("private.utils.target", {alias = "target_utils"})
 
 function _translate_path(dir, vcxprojdir)
     if dir == nil then
@@ -79,6 +81,11 @@ function _clear_cache()
     localcache.save()
 end
 
+-- get c++ modules rules
+function _get_cxxmodules_rules()
+    return {"c++.build.modules", "c++.build.modules.builder"}
+end
+
 -- get command string
 function _get_command_string(cmd, vcxprojdir)
     local kind = cmd.kind
@@ -116,94 +123,6 @@ function _get_command_string(cmd, vcxprojdir)
     end
 end
 
--- add target custom commands for target
-function _make_custom_commands_for_target(commands, target, vcxprojdir, suffix)
-    for _, ruleinst in ipairs(target:orderules()) do
-        local scriptname = "buildcmd" .. (suffix and ("_" .. suffix) or "")
-        local script = ruleinst:script(scriptname)
-        if script then
-            local batchcmds_ = batchcmds.new({target = target})
-            script(target, batchcmds_, {})
-            if not batchcmds_:empty() then
-                for _, cmd in ipairs(batchcmds_:cmds()) do
-                    local command = _get_command_string(cmd, vcxprojdir)
-                    if command then
-                        local key = suffix and suffix or "before"
-                        commands[key] = commands[key] or {}
-                        table.insert(commands[key], command)
-                    end
-                end
-            end
-        end
-
-        scriptname = "linkcmd" .. (suffix and ("_" .. suffix) or "")
-        script = ruleinst:script(scriptname)
-        if script then
-            local batchcmds_ = batchcmds.new({target = target})
-            script(target, batchcmds_, {})
-            if not batchcmds_:empty() then
-                for _, cmd in ipairs(batchcmds_:cmds()) do
-                    local command = _get_command_string(cmd, vcxprojdir)
-                    if command then
-                        local key = (suffix and suffix or "before") .. "_link"
-                        commands[key] = commands[key] or {}
-                        table.insert(commands[key], command)
-                    end
-                end
-            end
-        end
-    end
-end
-
--- add target custom commands for object rules
-function _make_custom_commands_for_objectrules(commands, target, sourcebatch, vcxprojdir, suffix)
-
-    -- get rule
-    local rulename = assert(sourcebatch.rulename, "unknown rule for sourcebatch!")
-    local ruleinst = assert(target:rule(rulename) or project.rule(rulename) or rule.rule(rulename), "unknown rule: %s", rulename)
-
-    -- generate commands for xx_buildcmd_files
-    local scriptname = "buildcmd_files" .. (suffix and ("_" .. suffix) or "")
-    local script = ruleinst:script(scriptname)
-    if script then
-        local batchcmds_ = batchcmds.new({target = target})
-        script(target, batchcmds_, sourcebatch, {})
-        if not batchcmds_:empty() then
-            for _, cmd in ipairs(batchcmds_:cmds()) do
-                local command = _get_command_string(cmd, vcxprojdir)
-                if command then
-                    local key = suffix and suffix or "before"
-                    commands[key] = commands[key] or {}
-                    table.insert(commands[key], command)
-                end
-            end
-        end
-    end
-
-    -- generate commands for xx_buildcmd_file
-    if not script then
-        scriptname = "buildcmd_file" .. (suffix and ("_" .. suffix) or "")
-        script = ruleinst:script(scriptname)
-        if script then
-            local sourcekind = sourcebatch.sourcekind
-            for _, sourcefile in ipairs(sourcebatch.sourcefiles) do
-                local batchcmds_ = batchcmds.new({target = target})
-                script(target, batchcmds_, sourcefile, {})
-                if not batchcmds_:empty() then
-                    for _, cmd in ipairs(batchcmds_:cmds()) do
-                        local command = _get_command_string(cmd, vcxprojdir)
-                        if command then
-                            local key = suffix and suffix or "before"
-                            commands[key] = commands[key] or {}
-                            table.insert(commands[key], command)
-                        end
-                    end
-                end
-            end
-        end
-    end
-end
-
 -- make custom commands
 function _make_custom_commands(target, vcxprojdir)
     -- https://github.com/xmake-io/xmake/issues/2337
@@ -212,20 +131,27 @@ function _make_custom_commands(target, vcxprojdir)
     target:data_set("plugin.project.translate_path", function (p)
         return _translate_path(p, vcxprojdir)
     end)
+
+    -- ignore c++ modules rules
+    local ignored_rules = _get_cxxmodules_rules()
+
+    -- add before commands
+    -- we use irpairs(groups), because the last group that should be given the highest priority.
+    -- rule.on_buildcmd_files should also be executed before building the target
+    local cmds_before = target_cmds.get_target_buildcmds(target, {ignored_rules = ignored_rules, stages = {"before", "on"}})
+
+    -- add after commands
+    local cmds_after = target_cmds.get_target_buildcmds(target, {ignored_rules = ignored_rules, stages = {"after"}})
+
     local commands = {}
-    _make_custom_commands_for_target(commands, target, vcxprojdir, "before")
-    _make_custom_commands_for_target(commands, target, vcxprojdir)
-    local sourcebatches = target:sourcebatches()
-    for _, sourcebatch in table.orderpairs(sourcebatches) do
-        local rulename = sourcebatch.rulename
-        local sourcekind = sourcebatch.sourcekind
-        if rulename ~= "c.build" and rulename ~= "c++.build" and rulename ~= "asm.build" and rulename ~= "cuda.build" and sourcekind ~= "mrc" then
-            _make_custom_commands_for_objectrules(commands, target, sourcebatch, vcxprojdir, "before")
-            _make_custom_commands_for_objectrules(commands, target, sourcebatch, vcxprojdir, nil)
-            _make_custom_commands_for_objectrules(commands, target, sourcebatch, vcxprojdir, "after")
-        end
+    for _, cmd in ipairs(cmds_before) do
+        commands.before = commands.before or {}
+        table.insert(commands.before, _get_command_string(cmd, vcxprojdir))
+    end
+    for _, cmd in ipairs(cmds_after) do
+        commands.after = commands.after or {}
+        table.insert(commands.after, _get_command_string(cmd, vcxprojdir))
     end
-    _make_custom_commands_for_target(commands, target, vcxprojdir, "after")
     return commands
 end
 
@@ -256,6 +182,9 @@ function _make_targetinfo(mode, arch, target, vcxprojdir)
     -- save symbols
     targetinfo.symbols = target:get("symbols")
 
+    -- has modules
+    targetinfo.has_modules = target:data("cxx.has_modules")
+
     -- save target kind
     targetinfo.targetkind = target:kind()
     if target:is_phony() or target:is_headeronly() then
@@ -268,9 +197,6 @@ function _make_targetinfo(mode, arch, target, vcxprojdir)
     -- save symbol file
     targetinfo.symbolfile = target:symbolfile()
 
-    -- save sourcebatches
-    targetinfo.sourcebatches = target:sourcebatches()
-
     -- save sourcekinds
     targetinfo.sourcekinds = target:sourcekinds()
 
@@ -289,9 +215,9 @@ function _make_targetinfo(mode, arch, target, vcxprojdir)
         local sourcekind = sourcebatch.sourcekind
         local rulename = sourcebatch.rulename
         if sourcekind then
-            for idx, sourcefile in ipairs(sourcebatch.sourcefiles) do
+            for _, sourcefile in ipairs(sourcebatch.sourcefiles) do
                 local compflags = compiler.compflags(sourcefile, {target = target, sourcekind = sourcekind})
-                if not firstcompflags and (rulename == "c.build" or rulename == "c++.build" or rulename == "cuda.build") then
+                if not firstcompflags and (rulename == "c.build" or rulename == "c++.build" or rulename == "c++.build.modules" or rulename == "cuda.build") then
                     firstcompflags = compflags
                 end
                 targetinfo.compflags[sourcefile] = compflags
@@ -309,8 +235,11 @@ function _make_targetinfo(mode, arch, target, vcxprojdir)
         end
     end
 
+    -- save sourcebatches
+    targetinfo.sourcebatches = target:sourcebatches()
+
     -- save linker flags
-    local linkflags = linker.linkflags(target:kind(), target:sourcekinds(), {target = target})
+    local linkflags = linker.linkflags(target:is_moduleonly() and 'static' or target:kind(), target:sourcekinds(), {target = target})
     targetinfo.linkflags = linkflags
 
     if table.contains(target:sourcekinds(), "cu") then
@@ -323,7 +252,7 @@ function _make_targetinfo(mode, arch, target, vcxprojdir)
     end
 
     -- save execution dir (when executed from VS)
-    targetinfo.rundir = target:rundir()
+    targetinfo.rundir = target:is_moduleonly() and "" or target:rundir()
 
     -- save runenvs
     local targetrunenvs = {}
@@ -449,17 +378,14 @@ end
 
 -- make vstudio project
 function make(outputdir, vsinfo)
-
-    -- enter project directory
     local oldir = os.cd(project.directory())
 
-    -- init solution directory
+    -- prepare targets
+    target_cmds.prepare_targets()
+
+    -- init vsinfo
     vsinfo.solution_dir = path.join(outputdir, "vs" .. vsinfo.vstudio_version)
-
-    -- init modes
     vsinfo.modes = _make_vsinfo_modes()
-
-    -- init archs
     vsinfo.archs = _make_vsinfo_archs()
 
     -- load targets
@@ -479,7 +405,7 @@ function make(outputdir, vsinfo)
                 config.set("arch", arch, {readonly = true, force = true})
 
                 -- clear all options
-                for _, opt in ipairs(project.options()) do
+                for _, opt in pairs(project.options()) do
                     opt:clear()
                 end
 
@@ -511,7 +437,8 @@ function make(outputdir, vsinfo)
             os.cd(project.directory())
 
             -- save targets
-            for targetname, target in table.orderpairs(project.targets()) do
+            local project_targets = target_utils.get_project_targets()
+            for targetname, target in table.orderpairs(project_targets) do
 
                 -- make target with the given mode and arch
                 targets[targetname] = targets[targetname] or {}
@@ -544,6 +471,13 @@ function make(outputdir, vsinfo)
                 -- save file groups
                 _target.filegroups = table.unique(table.join(_target.filegroups or {}, target:get("filegroups")))
 
+                -- save references to deps
+                for _, dep in ipairs(target:orderdeps()) do
+                    _target.deps = _target.deps or {}
+                    local dep_name = dep:name()
+                    _target.deps[dep_name] = path.relative(path.join(vsinfo.solution_dir, dep_name, dep_name .. ".vcxproj"), _target.project_dir)
+                end
+
                 for filegroup, groupconf in pairs(target:extraconf("filegroups")) do
                     _target.filegroups_extraconf = _target.filegroups_extraconf or {}
                     local mergedconf = _target.filegroups_extraconf[filegroup]
@@ -575,7 +509,5 @@ function make(outputdir, vsinfo)
 
     -- clear local cache
     _clear_cache()
-
-    -- leave project directory
     os.cd(oldir)
 end
